require(tidyverse)
require(hrbrthemes)
require(directlabels)

# GGplot theme for all the graphs
theme_bmbp <- function(...) {
    theme_ipsum(base_family = "Helvetica", axis_text_size = 8)
}

# Common-Language effect size 
CLES <- function(x, y, SWC = 0, na.rm = FALSE) {
    combinations <- expand.grid(x = x, y = y)
    differences <- combinations$y - combinations$x
    proportion <- sum(ifelse(differences == SWC, 0.5, differences > SWC), na.rm = na.rm)/length(differences)
    return(proportion)   
}

set.seed(6667)
sampleA <- rnorm(n = 1000, mean = 40, sd = 8)

# --------------------- Exact same sample
CLES(sampleA, sampleA, SWC = 0) 

CLES(sampleA, sampleA, SWC = 5) 

# ---------------------- Increase for d = 0.2 (common SWC)
set.seed(6667)
sampleB <- rnorm(n = 1000, mean = 40 + (sd(sampleA) * 0.2), sd = 8)

CLES(sampleA, sampleB, SWC = 0) 

CLES(sampleA, sampleB, SWC = 5) 

# ---------------------- Create a function
cohenD <- seq(0, 3, length.out = 10)
swc <- 0:10

parameters <- expand.grid(d = cohenD, swc = swc)
parameters$cles <- NA

for (i in seq(1, nrow(parameters))) {
    print(i)
    set.seed(6667)
    parameters$cles[i] <- CLES(sampleA, 
                               rnorm(n = 1000, mean = 40 + (sd(sampleA) * parameters$d[i]), sd = 8),
                               SWC = parameters$swc[i])     
}

gg <- ggplot(parameters, aes(x = d, y = cles, group = swc)) +
    theme_bmbp() +
    geom_hline(yintercept = 0.5, color = "dark grey") +
    geom_hline(yintercept = 0.6, color = "dark grey", linetype = "dashed") +
    geom_line() +
    geom_dl(aes(label = swc),
              method = list(dl.trans(x = x - .2), cex = 0.5, "first.points"), color = "black") 

    
gg

# ------------------------ model using GAM
require(gam)

model <- gam(d ~ s(swc) + s(cles), data = parameters)

forPrediction <- expand.grid(swc = 0:10, cles = 0.6)
cohenD <- predict(model, newdata = forPrediction)
forPrediction$d <- cohenD
forPrediction$swc <- factor(forPrediction$swc)

gg <- ggplot(forPrediction, aes(x = swc, y = d)) +
    theme_bmbp() +
    geom_bar(stat = "identity", alpha = 0.8 ) +
    geom_text(aes(x = swc, y = d + 0.05, label = round(d, 2)), size = 2) +
    ggtitle(label = NULL, subtitle = "Needed Cohen's D to reach CLES of 60%")
gg
